@@ -60,7 +60,7 @@ for fn in glob.glob("%s/*.c" % sys.argv[1]):
 	line = 1
 	nfeat = 0
 	for s in f.readlines():
-		if rexp.match(s):
+		if rexp.match(s) and "FEATURE_LEVEL_" not in s:
 			nfeat += 1
 			res = rexp.findall(s)
 			assert len(res) == 1, "Too many matches"
@@ -108,18 +108,30 @@ firstError = True
 
 files_code = {}
 
-rexp = re.compile('\t+{ *[0-9]+, (CPU_FEATURE_[^ }]+).*')
+rexp1 = re.compile(r'.*\[(CPU_FEATURE_[^ \]]+)\]\s*=\s*{.*') # e.g. "[CPU_FEATURE_SWAP] = {"
+rexp2 = re.compile(r'.*(CPU_FEATURE_[^ ,]+),\s*FEATURE_LEVEL_ARM_.*') # e.g. "set_feature_status(data, ext_status, (mte_frac == 0b0000), CPU_FEATURE_MTE_ASYNC,           FEATURE_LEVEL_ARM_V8_5_A, -1);"
+rexp3 = re.compile(r'.*(CPU_FEATURE_[^ }]+).*') # e.g. "{ 28, CPU_FEATURE_HT },"
 
 for fn in glob.glob("%s/*.c" % sys.argv[1]):
 	f = open(fn, "rt")
 	files_code[fn] = []
 	for s in f.readlines():
-		if rexp.match(s):
-			entry = rexp.findall(s)[0]
+		if rexp1.match(s):
+			entry = rexp1.findall(s)[0]
 			files_code[fn].append(entry)
+		elif rexp2.match(s):
+			entry = rexp2.findall(s)[0]
+			files_code[fn].append(entry)
+		elif rexp3.match(s):
+			entry = rexp3.findall(s)[0]
+			files_code[fn].append(entry)
+
 	f.close()
 
-features_whitelist = ["CPU_FEATURE_SSE5"]
+features_whitelist = [
+	"CPU_FEATURE_SSE5", # deprecated, never defined
+	"CPU_FEATURE_AES", # defined twice (x86 + ARM)
+]
 for feature in allf:
 	matching_files = []
 	for fn in files_code:
@@ -131,7 +143,7 @@ for feature in allf:
 			firstError = False
 		err += 1
 		print("..No detection code for %s" % feature)
-	if len(matching_files) > 1:
+	if len(matching_files) > 1 and feature not in features_whitelist:
 		if firstError:
 			print("FAILED:")
 			firstError = False
@@ -143,10 +155,13 @@ if firstError:
 print("")
 
 print("Checking processor definitions:")
-cache_exp = re.compile(".*([\(/ ][0-9]+K).*")
+cache_exp = re.compile(r".*([\(/ ][0-9]+K).*")
 # Check whether CPU codenames for consistency:
 #   - Codenames should not exceed 31 characters
 #   - Check for common typos
+definitions = 0
+match_entry_fields = 12 # this number needs to change if the definition of match_entry_t ever changes
+codename_str_max = 64-1 # this number needs to change if the value of CODENAME_STR_MAX ever changes
 common_cache_sizes = ["8", "16", "32", "64", "128", "256", "512", "1024", "2048", "3072", "4096", "6144", "8192", "12288", "16384"]
 for fn in glob.glob("%s/*.c" % sys.argv[1]):
 	bfn = os.path.basename(fn)
@@ -167,18 +182,18 @@ for fn in glob.glob("%s/*.c" % sys.argv[1]):
 			continue
 		inner = line[i+1:j]
 		parts = inner.split(",")
-		if len(parts) == 11: #this number needs to change if the definition of match_entry_t ever changes
+		if len(parts) == match_entry_fields:
 			cdefs += 1
-			s = parts[10].strip()
+			s = parts[match_entry_fields-1].strip()
 			if s[0] != '"' or s[-1] != '"':
 				err += 1
 				print("..Warning, %s:%d - cannot correctly handle the cpu codename" % (bfn, nline))
 				allok = False
 				continue
 			s = s[1:-1]
-			if len(s) > 31:
+			if len(s) > codename_str_max:
 				err += 1
-				print("..%s:%d - codename (%s) is longer than 31 characters!" % (bfn, nline, s))
+				print("..%s:%d - codename (%s) is longer than %d characters!" % (bfn, nline, s, codename_str_max))
 				allok = False
 			if cache_exp.match(s):
 				cache_size = cache_exp.findall(s)[0][1:-1]
@@ -187,11 +202,15 @@ for fn in glob.glob("%s/*.c" % sys.argv[1]):
 					print("..Warning, %s:%d - suspicious cache size in codename [%s] (%s)" % (bfn, nline, s, cache_size))
 					allok = False
 	if cdefs:
+		definitions += 1
 		print("  %s: %d processor definitions," % (bfn, cdefs), end=' ')
 		if allok:
 			print("all OK")
 		else:
 			print("some errors/warnings")
 	f.close()
+if definitions == 0:
+	err += 1
+	print("..Warning, no processor definitions found")
 
 sys.exit(err)
