@@ -12,7 +12,13 @@ if len(args) != 3:
 def readRawFile():
 	rawdata = []
 	for line in open(args[1], "rt").readlines():
-		lookfor = ["basic_cpuid", "ext_cpuid", "intel_fn4", "intel_fn11", "amd_fn8000001dh", "Logical CPU", "CPUID", "CPU#"]
+		lookfor = [
+			"Logical CPU", "CPUID", "CPU#", # common
+			"basic_cpuid", "ext_cpuid", "intel_fn4", "intel_fn11", "amd_fn8000001dh", # x86
+			"arm_midr", "arm_mpidr", "arm_revidr", # ARM common
+			"arm_id_afr", "arm_id_dfr", "arm_id_isar", "arm_id_mmfr", "arm_id_pfr", # ARM (AArch32)
+			"arm_id_aa64afr", "arm_id_aa64dfr", "arm_id_aa64isar", "arm_id_aa64mmfr", "arm_id_aa64pfr", "arm_id_aa64smfr", "arm_id_aa64zfr" # ARM (AArch64)
+		]
 		ignore  = ["MSR Register"]
 		good = False
 		for match in lookfor:
@@ -35,6 +41,7 @@ def readResultFile():
 		if s.find(":") == -1:
 			continue
 		numeric = ["family", "model", "stepping", "ext_family", "ext_model",
+				"implementer", "variant", "part_num", "revision",
 				"num_cores", "num_logical",
 				"L1 D cache",  "L1 I cache",  "L2 cache",   "L3 cache",   "L4 cache",
 				"L1D assoc.",  "L1I assoc.",  "L2 assoc.",  "L3 assoc.",  "L4 assoc.",
@@ -52,6 +59,9 @@ def readResultFile():
 		if field == "arch":
 			value = s[s.find(":") + 2:]
 			repdata.append(value)
+		if field == "feat_level":
+			value = s[s.find(":") + 2:]
+			repdata.append(value)
 		if field == "purpose":
 			value = s[s.find(":") + 2:]
 			repdata.append(value)
